"""Module to manage records and their logs for the StepTracer library."""
import json
import shutil
import tempfile
import traceback
from datetime import datetime
from pathlib import Path
from typing import Optional

from .models.models import RecordStatus
from .models.records_logs import RecordLog, RecordsLogs
from .models.status_update import StatusUpdate
from .models.trace import Trace
from .remote_storage import RemoteStorage
from .report.html_generator import generate_enhanced_html
from .utils.utils import compare_dicts, format_traceback_short


class RecordsManager:
    """Singleton class to manage records and their logs."""

    _instance = None

    def __new__(cls, *args, **kwargs):
        """Singleton pattern to ensure only one instance of RecordsManager exists."""
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, root_folder: str | Path):
        """Initialize the RecordsManager with a root folder."""
        self.storage: RemoteStorage = RemoteStorage(root_folder)
        self.global_records_logs: RecordsLogs = self.storage.get_records_logs()
        self.local_records_logs: RecordsLogs = RecordsLogs()
        self.temp_dir = tempfile.TemporaryDirectory()

    def dump(self) -> None:
        """Dump the local records logs to the remote storage."""
        self.storage.update_records_logs(self.local_records_logs)
        merged_logs = self.storage.get_records_logs()
        self.storage.save_html(generate_enhanced_html(merged_logs.data))
        self.temp_dir.cleanup()

    def pack_sharable_zip(self, file_path: str) -> None:
        """Pack all records logs and HTML report into a zip file."""
        self.storage.get_zip(file_path)

    def is_record_registered(self, record_id: str) -> bool:
        """Check if a record is already registered."""
        return record_id in self.global_records_logs.data or record_id in self.local_records_logs.data

    def register_record(self, record: "ThoughtfulRecord") -> None:  # noqa: F821
        """Register a new record and initialize its log."""
        current_dict = json.loads(record.to_json_str())
        dict_updates = compare_dicts({}, current_dict)

        trace = Trace(
            action="Initialize record",
            timestamp=datetime.now(),
            dict_updates=dict_updates,
            traceback=format_traceback_short(traceback.extract_stack()[:-2]),
        )

        new_log = RecordLog(record_id=record.id, record=record.to_json_str(), traces=[trace])

        if record._status_field:
            record_status: RecordStatus = getattr(record, record._status_field)
            if isinstance(record_status, str):
                record_status = RecordStatus(record_status)
            new_log.status = record_status
            new_log.status_color = record_status.color

        self.local_records_logs.data[record.id] = new_log

    def get_or_prepare_local_record_log(self, record: "ThoughtfulRecord") -> RecordLog:  # noqa: F821
        """Get record_log for record, ensure it is present in local_records_logs."""
        if record.id in self.local_records_logs.data:
            return self.local_records_logs.data[record.id]

        if record.id in self.global_records_logs.data:
            global_log = self.global_records_logs.data[record.id]

            # Create local log copy — only essential fields
            local_log = RecordLog(
                record_id=global_log.record_id,
                record=global_log.record,
                status=global_log.status,
                status_color=global_log.status_color,
                status_updates=[],
                traces=[],
            )

            self.local_records_logs.data[record.id] = local_log
            return local_log

        raise RuntimeError(f"Record {record.id} is not registered — this should not happen!")

    def update_status(self, record: "ThoughtfulRecord", new_status: RecordStatus) -> None:  # noqa: F821
        """Update the status of a record and log the change."""
        record_log = self.get_or_prepare_local_record_log(record)

        status_update = StatusUpdate(
            timestamp=datetime.now(),
            old_status=record_log.status,
            old_status_color=record_log.status_color,
            new_status=new_status,
            new_status_color=new_status.color,
            traceback=format_traceback_short(traceback.extract_stack()[:-2]),
        )

        record_log.status_updates.append(status_update)
        record_log.status = new_status
        record_log.status_color = new_status.color

    def log_trace(
        self, record: "ThoughtfulRecord", action: str, reason: str, image: Optional[str] = ""  # noqa: F821
    ) -> None:
        """Log a trace for a record with the specified action, reason, and optional image."""
        record_log = self.get_or_prepare_local_record_log(record)

        previous_dict = json.loads(record_log.record)
        current_dict = json.loads(record.to_json_str())
        record_log.record = record.to_json_str()

        dict_updates = compare_dicts(previous_dict, current_dict)

        if image:
            unique_image_name = f"{record.id}_{datetime.now().strftime('%Y%m%d_%H%M%S')}{Path(image).suffix}".replace(
                " ", "_"
            )
            dest = Path(self.temp_dir.name) / unique_image_name
            dest.parent.mkdir(parents=True, exist_ok=True)
            image = shutil.copy(image, dest)

        trace = Trace(
            action=action,
            reason=reason,
            timestamp=datetime.now(),
            dict_updates=dict_updates,
            image=str(image),
            traceback=format_traceback_short(traceback.extract_stack()[:-2]),
        )

        record_log.traces.append(trace)
